@@ -3,6 +3,7 @@ from normality import slugify
 from datetime import datetime
 from banal import ensure_list
 from followthemoney import model
+from followthemoney.proxy import EntityProxy
 from sqlalchemy import Column, DateTime, String, UniqueConstraint
 from sqlalchemy import Table, MetaData, JSON
 from sqlalchemy import create_engine, select, distinct, func
@@ -10,37 +11,46 @@ from sqlalchemy.dialects.postgresql import JSONB
 
 from ftmstore import settings
 from ftmstore.loader import BulkLoader
+from ftmstore.utils import DroppedException
 
-NULL_ORIGIN = 'null'
+NULL_ORIGIN = "null"
 log = logging.getLogger(__name__)
 
 
 class Dataset(object):
-
-    def __init__(self, name, origin=NULL_ORIGIN,
-                 database_uri=settings.DATABASE_URI,
-                 prefix=settings.DATABASE_PREFIX,
-                 **config):
+    def __init__(
+        self,
+        name,
+        origin=NULL_ORIGIN,
+        database_uri=settings.DATABASE_URI,
+        prefix=settings.DATABASE_PREFIX,
+        **config
+    ):
         self.name = name
         self.origin = origin
         self.prefix = prefix
         # config.setdefault('pool_size', 1)
         self.engine = create_engine(database_uri, **config)
-        self.is_postgres = self.engine.dialect.name == 'postgresql'
+        self.is_postgres = self.engine.dialect.name == "postgresql"
         meta = MetaData(self.engine)
-        table_name = slugify('%s %s' % (self.prefix, self.name), sep='_')
-        self.table = Table(table_name, meta,
-            Column('id', String, nullable=False),  # noqa
-            Column('origin', String, nullable=False),
-            Column('fragment', String, nullable=False),
-            Column('timestamp', DateTime, default=datetime.utcnow),
-            Column('entity', JSONB if self.is_postgres else JSON),
-            UniqueConstraint('id', 'origin', 'fragment'),
-            extend_existing=True
+        table_name = slugify("%s %s" % (self.prefix, self.name), sep="_")
+        self.table = Table(
+            table_name,
+            meta,
+            Column("id", String, nullable=False),
+            Column("origin", String, nullable=False),
+            Column("fragment", String, nullable=False),
+            Column("timestamp", DateTime, default=datetime.utcnow),
+            Column("entity", JSONB if self.is_postgres else JSON),
+            UniqueConstraint("id", "origin", "fragment"),
+            extend_existing=True,
         )
         self.table.create(bind=self.engine, checkfirst=True)
+        self._dropped = False
 
     def delete(self, entity_id=None, fragment=None, origin=None):
+        if self._dropped:
+            raise DroppedException()
         table = self.table
         stmt = table.delete()
         if entity_id is not None:
@@ -52,6 +62,8 @@ class Dataset(object):
         self.engine.execute(stmt)
 
     def drop(self):
+        self._dropped = True
+        log.debug("Dropping ftm-store: %s", self.table)
         self.table.drop(self.engine)
         self.close()
 
@@ -67,6 +79,8 @@ class Dataset(object):
         self.engine.dispose()
 
     def fragments(self, entity_ids=None, fragment=None):
+        if self._dropped:
+            raise DroppedException()
         stmt = self.table.select()
         entity_ids = ensure_list(entity_ids)
         if len(entity_ids) == 1:
@@ -82,26 +96,36 @@ class Dataset(object):
         try:
             conn = conn.execution_options(stream_results=True)
             for ent in conn.execute(stmt):
-                data = {'id': ent.id, **ent.entity}
+                data = {"id": ent.id, **ent.entity}
                 if ent.origin != NULL_ORIGIN:
-                    data['origin'] = ent.origin
+                    data["origin"] = ent.origin
                 yield data
         finally:
             conn.close()
 
     def partials(self, entity_id=None):
         for fragment in self.fragments(entity_ids=entity_id):
-            yield model.get_proxy(fragment)
+            yield EntityProxy(model, fragment, cleaned=True)
 
     def iterate(self, entity_id=None):
         entity = None
+        fragments = 1
         for partial in self.partials(entity_id=entity_id):
             if entity is not None:
                 if entity.id == partial.id:
+                    fragments += 1
+                    if fragments % 10000 == 0:
+                        log.debug(
+                            "[%s:%s] aggregated %d fragments...",
+                            entity.schema.name,
+                            entity.id,
+                            fragments,
+                        )
                     entity.merge(partial)
                     continue
                 yield entity
             entity = partial
+            fragments = 1
         if entity is not None:
             yield entity
 
@@ -113,8 +137,10 @@ class Dataset(object):
         return self.iterate()
 
     def __len__(self):
+        if self._dropped:
+            raise DroppedException()
         q = select([func.count(distinct(self.table.c.id))])
         return self.engine.execute(q).scalar()
 
     def __repr__(self):
-        return '<Dataset(%r, %r)>' % (self.engine, self.name)
+        return "<Dataset(%r, %r)>" % (self.engine, self.name)
